@@ -8,7 +8,8 @@ from pytorch_metric_learning.distances import DotProductSimilarity
 from pytorch_metric_learning.losses import SupConLoss
 from tqdm.auto import tqdm
 
-from .data_utils.datasets import RowDataset
+from .data_utils import utils
+from .data_utils.datasets import RecordDataset
 from .early_stopping import EarlyStoppingMinEpochs, ModelCheckpointMinEpochs
 from .evaluation import f1_score, pair_entity_ratio, precision_and_recall
 from .indexes import ANNEntityIndex, ANNLinkageIndex
@@ -20,7 +21,7 @@ logger = logging.getLogger(__name__)
 class _BaseEmbed(pl.LightningModule):
     def __init__(
         self,
-        row_numericalizer,
+        record_numericalizer,
         embedding_size=300,
         loss_cls=SupConLoss,
         loss_kwargs=None,
@@ -38,8 +39,8 @@ class _BaseEmbed(pl.LightningModule):
         super().__init__()
         self.save_hyperparameters()
 
-        self.row_numericalizer = row_numericalizer
-        for field_config in self.row_numericalizer.field_config_dict.values():
+        self.record_numericalizer = record_numericalizer
+        for field_config in self.record_numericalizer.field_config_dict.values():
             vocab = field_config.vocab
             if vocab:
                 # We can assume that there's only one vocab type across the
@@ -53,7 +54,7 @@ class _BaseEmbed(pl.LightningModule):
                     )
         self.embedding_size = embedding_size
         self.blocker_net = BlockerNet(
-            field_config_dict=self.row_numericalizer.field_config_dict,
+            field_config_dict=self.record_numericalizer.field_config_dict,
             embedding_size=self.embedding_size,
         )
         self.loss_fn = loss_cls(**loss_kwargs if loss_kwargs else {"temperature": 0.1})
@@ -71,8 +72,11 @@ class _BaseEmbed(pl.LightningModule):
         self.index_build_kwargs = index_build_kwargs
         self.index_search_kwargs = index_search_kwargs
 
-    def forward(self, tensor_dict, sequence_length_dict):
-        return self.blocker_net(tensor_dict, sequence_length_dict)
+    def forward(self, tensor_dict, sequence_length_dict, return_field_embeddings=False):
+        tensor_dict = utils.tensor_dict_to_device(tensor_dict, device=self.device)
+        sequence_length_dict = utils.tensor_dict_to_device(sequence_length_dict, device=self.device)
+
+        return self.blocker_net(tensor_dict, sequence_length_dict, return_field_embeddings)
 
     def _warn_if_empty_indices_tuple(self, indices_tuple, batch_idx):
         with torch.no_grad():
@@ -112,7 +116,7 @@ class _BaseEmbed(pl.LightningModule):
     def validation_epoch_end(self, outputs):
         metric_dict = self._evaluate_with_ann(
             set_name="valid",
-            row_dict=self.trainer.datamodule.valid_row_dict,
+            record_dict=self.trainer.datamodule.valid_record_dict,
             embedding_batch_list=outputs,
             pos_pair_set=self.trainer.datamodule.valid_pos_pair_set,
         )
@@ -125,7 +129,7 @@ class _BaseEmbed(pl.LightningModule):
     def test_epoch_end(self, outputs):
         metric_dict = self._evaluate_with_ann(
             set_name="test",
-            row_dict=self.trainer.datamodule.test_row_dict,
+            record_dict=self.trainer.datamodule.test_record_dict,
             embedding_batch_list=outputs,
             pos_pair_set=self.trainer.datamodule.test_pos_pair_set,
         )
@@ -207,22 +211,22 @@ class _BaseEmbed(pl.LightningModule):
         )
         self.blocker_net = None
         best_model = self.load_from_checkpoint(trainer.checkpoint_callback.best_model_path)
+        best_model = best_model.to(self.device)
         self.blocker_net = best_model.blocker_net
         return trainer
 
-    def _evaluate_with_ann(self, set_name, row_dict, embedding_batch_list, pos_pair_set):
+    def _evaluate_with_ann(self, set_name, record_dict, embedding_batch_list, pos_pair_set):
         raise NotImplementedError
 
-    def _evaluate_metrics(self, set_name, dataloader, row_dict, pos_pair_set):
+    def _evaluate_metrics(self, set_name, dataloader, record_dict, pos_pair_set):
         embedding_batch_list = []
         for tensor_dict, sequence_length_dict in dataloader:
-            tensor_dict = {field: t.to(self.device) for field, t in tensor_dict.items()}
             embeddings = self(tensor_dict, sequence_length_dict)
             embedding_batch_list.append(embeddings)
 
         metric_dict = self._evaluate_with_ann(
             set_name=set_name,
-            row_dict=row_dict,
+            record_dict=record_dict,
             embedding_batch_list=embedding_batch_list,
             pos_pair_set=pos_pair_set,
         )
@@ -234,7 +238,7 @@ class _BaseEmbed(pl.LightningModule):
         metric_dict = self._evaluate_metrics(
             set_name="valid",
             dataloader=datamodule.val_dataloader(),
-            row_dict=datamodule.valid_row_dict,
+            record_dict=datamodule.valid_record_dict,
             pos_pair_set=datamodule.valid_pos_pair_set,
         )
         return metric_dict
@@ -247,52 +251,110 @@ class _BaseEmbed(pl.LightningModule):
         metric_dict = self._evaluate_metrics(
             set_name="test",
             dataloader=datamodule.test_dataloader(),
-            row_dict=datamodule.test_row_dict,
+            record_dict=datamodule.test_record_dict,
             pos_pair_set=datamodule.test_pos_pair_set,
         )
         return metric_dict
 
-    def predict(
-        self,
-        row_dict,
-        batch_size,
-        loader_kwargs=None,
-        show_progress=True,
-    ):
-        self.freeze()
-
-        row_dataset = RowDataset(
-            row_numericalizer=self.row_numericalizer, row_dict=row_dict, batch_size=batch_size
+    def _get_record_loader(self, record_dict, batch_size, loader_kwargs):
+        record_dataset = RecordDataset(
+            record_numericalizer=self.record_numericalizer,
+            record_dict=record_dict,
+            batch_size=batch_size,
         )
-        row_loader = torch.utils.data.DataLoader(
-            row_dataset,
-            batch_size=None,  # batch size is set on RowDataset
+        record_loader = torch.utils.data.DataLoader(
+            record_dataset,
+            batch_size=None,  # batch size is set on RecordDataset
             shuffle=False,
             **loader_kwargs
             if loader_kwargs
             else {"num_workers": os.cpu_count(), "multiprocessing_context": "fork"},
         )
+        return record_loader
+
+    def predict(
+        self,
+        record_dict,
+        batch_size,
+        loader_kwargs=None,
+        show_progress=True,
+        return_field_embeddings=False,
+    ):
+        self.freeze()
+        record_loader = self._get_record_loader(record_dict, batch_size, loader_kwargs)
 
         with tqdm(
-            total=len(row_loader), desc="# batch embedding", disable=not show_progress
+            total=len(record_loader), desc="# batch embedding", disable=not show_progress
         ) as p_bar:
             vector_list = []
-            for tensor_dict, sequence_length_dict in row_loader:
-                tensor_dict = {field: t.to(self.device) for field, t in tensor_dict.items()}
-                embeddings = self(tensor_dict, sequence_length_dict)
-                vector_list.extend(v.data.numpy() for v in embeddings.cpu().unbind())
+            if return_field_embeddings:
+                field_vector_dict = {
+                    field: [] for field in self.blocker_net.field_config_dict.keys()
+                }
+            else:
+                field_vector_dict = None
+
+            for tensor_dict, sequence_length_dict in record_loader:
+                result = self(tensor_dict, sequence_length_dict, return_field_embeddings)
+                if return_field_embeddings:
+                    field_embeddings_dict, avg_embeddings = result
+                    for field, field_embeddings in field_embeddings_dict.items():
+                        field_vector_dict[field].extend(
+                            v.data.numpy() for v in field_embeddings.cpu().unbind()
+                        )
+                else:
+                    avg_embeddings = result
+
+                vector_list.extend(v.data.numpy() for v in avg_embeddings.cpu().unbind())
                 p_bar.update(1)
 
-        vector_dict = dict(zip(row_dict.keys(), vector_list))
-        return vector_dict
+        vector_dict = dict(zip(record_dict.keys(), vector_list))
+        if return_field_embeddings:
+            field_vector_dict = {
+                id_: dict(zip(field_vector_dict.keys(), field_values))
+                for id_, *field_values in zip(record_dict.keys(), *field_vector_dict.values())
+            }
+            return vector_dict, field_vector_dict
+        else:
+            return vector_dict
+
+    def interpret_attention(
+        self,
+        record_dict,
+        batch_size,
+        field,
+        loader_kwargs=None,
+        show_progress=True,
+    ):
+        self.freeze()
+        record_loader = self._get_record_loader(record_dict, batch_size, loader_kwargs)
+
+        with tqdm(
+            total=len(record_loader), desc="# batch embedding", disable=not show_progress
+        ) as p_bar:
+            attn_scores_list = []
+
+            for tensor_dict, sequence_length_dict in record_loader:
+                field_tensor = tensor_dict[field].to(self.device)
+                sequence_lengths = sequence_length_dict[field].to(self.device)
+
+                (__, attn_scores) = self.blocker_net.field_embed_net.embed_net_dict[field]._forward(
+                    field_tensor, sequence_lengths
+                )
+
+                attn_scores_list.extend(v.data.numpy() for v in attn_scores.cpu().unbind())
+                p_bar.update(1)
+
+        attn_scores_dict = dict(zip(record_dict.keys(), attn_scores_list))
+        return attn_scores_dict
 
 
 class EntityEmbed(_BaseEmbed):
-    def _evaluate_with_ann(self, set_name, row_dict, embedding_batch_list, pos_pair_set):
+    def _evaluate_with_ann(self, set_name, record_dict, embedding_batch_list, pos_pair_set):
         vector_list = []
         for embedding_batch in embedding_batch_list:
             vector_list.extend(v.data.numpy() for v in embedding_batch.cpu().unbind())
-        vector_dict = dict(zip(row_dict.keys(), vector_list))
+        vector_dict = dict(zip(record_dict.keys(), vector_list))
 
         ann_index = ANNEntityIndex(embedding_size=self.embedding_size)
         ann_index.insert_vector_dict(vector_dict)
@@ -322,7 +384,7 @@ class EntityEmbed(_BaseEmbed):
 
     def predict_pairs(
         self,
-        row_dict,
+        record_dict,
         batch_size,
         ann_k,
         sim_threshold,
@@ -330,26 +392,37 @@ class EntityEmbed(_BaseEmbed):
         index_build_kwargs=None,
         index_search_kwargs=None,
         show_progress=True,
+        return_field_embeddings=False,
     ):
-        vector_dict = self.predict(
-            row_dict=row_dict,
+        result = self.predict(
+            record_dict=record_dict,
             batch_size=batch_size,
             loader_kwargs=loader_kwargs,
             show_progress=show_progress,
+            return_field_embeddings=return_field_embeddings,
         )
+        if return_field_embeddings:
+            vector_dict, field_vector_dict = result
+        else:
+            vector_dict = result
+
         ann_index = ANNEntityIndex(embedding_size=self.embedding_size)
         ann_index.insert_vector_dict(vector_dict)
         ann_index.build(index_build_kwargs=index_build_kwargs)
         found_pair_set = ann_index.search_pairs(
             k=ann_k, sim_threshold=sim_threshold, index_search_kwargs=index_search_kwargs
         )
-        return found_pair_set
+
+        if return_field_embeddings:
+            return found_pair_set, field_vector_dict
+        else:
+            return found_pair_set
 
 
 class LinkageEmbed(_BaseEmbed):
     def __init__(
         self,
-        row_numericalizer,
+        record_numericalizer,
         source_field,
         left_source,
         **kwargs,
@@ -358,20 +431,20 @@ class LinkageEmbed(_BaseEmbed):
         self.left_source = left_source
 
         super().__init__(
-            row_numericalizer=row_numericalizer,
+            record_numericalizer=record_numericalizer,
             source_field=source_field,
             left_source=left_source,
             **kwargs,
         )
 
-    def _evaluate_with_ann(self, set_name, row_dict, embedding_batch_list, pos_pair_set):
+    def _evaluate_with_ann(self, set_name, record_dict, embedding_batch_list, pos_pair_set):
         vector_list = []
         for embedding_batch in embedding_batch_list:
             vector_list.extend(v.data.numpy() for v in embedding_batch.cpu().unbind())
         left_vector_dict = {}
         right_vector_dict = {}
-        for (id_, row), vector in zip(row_dict.items(), vector_list):
-            if row[self.source_field] == self.left_source:
+        for (id_, record), vector in zip(record_dict.items(), vector_list):
+            if record[self.source_field] == self.left_source:
                 left_vector_dict[id_] = vector
             else:
                 right_vector_dict[id_] = vector
@@ -409,31 +482,58 @@ class LinkageEmbed(_BaseEmbed):
         metric_dict = dict(sorted(metric_dict.items(), key=lambda kv: kv[0]))
         return metric_dict
 
-    def predict(
-        self,
-        row_dict,
-        batch_size,
-        loader_kwargs=None,
-        show_progress=True,
-    ):
-        vector_dict = super().predict(
-            row_dict=row_dict,
-            batch_size=batch_size,
-            loader_kwargs=loader_kwargs,
-            show_progress=show_progress,
-        )
+    def _separate_left_right_vector_dict(self, record_dict, vector_dict):
         left_vector_dict = {}
         right_vector_dict = {}
-        for (id_, row), vector in zip(row_dict.items(), vector_dict.values()):
-            if row[self.source_field] == self.left_source:
+
+        for (id_, record), vector in zip(record_dict.items(), vector_dict.values()):
+            if record[self.source_field] == self.left_source:
                 left_vector_dict[id_] = vector
             else:
                 right_vector_dict[id_] = vector
+
         return left_vector_dict, right_vector_dict
 
+    def predict(
+        self,
+        record_dict,
+        batch_size,
+        loader_kwargs=None,
+        show_progress=True,
+        return_field_embeddings=False,
+    ):
+        result = super().predict(
+            record_dict=record_dict,
+            batch_size=batch_size,
+            loader_kwargs=loader_kwargs,
+            show_progress=show_progress,
+            return_field_embeddings=return_field_embeddings,
+        )
+        if return_field_embeddings:
+            vector_dict, field_vector_dict = result
+            left_field_vector_dict, right_field_vector_dict = self._separate_left_right_vector_dict(
+                record_dict, field_vector_dict
+            )
+        else:
+            vector_dict = result
+
+        left_vector_dict, right_vector_dict = self._separate_left_right_vector_dict(
+            record_dict, vector_dict
+        )
+
+        if return_field_embeddings:
+            return (
+                left_vector_dict,
+                right_vector_dict,
+                left_field_vector_dict,
+                right_field_vector_dict,
+            )
+        else:
+            return left_vector_dict, right_vector_dict
+
     def predict_pairs(
         self,
-        row_dict,
+        record_dict,
         batch_size,
         ann_k,
         sim_threshold,
@@ -441,13 +541,25 @@ class LinkageEmbed(_BaseEmbed):
         index_build_kwargs=None,
         index_search_kwargs=None,
         show_progress=True,
+        return_field_embeddings=False,
     ):
-        left_vector_dict, right_vector_dict = self.predict(
-            row_dict=row_dict,
+        result = self.predict(
+            record_dict=record_dict,
             batch_size=batch_size,
             loader_kwargs=loader_kwargs,
             show_progress=show_progress,
+            return_field_embeddings=return_field_embeddings,
         )
+        if return_field_embeddings:
+            (
+                left_vector_dict,
+                right_vector_dict,
+                left_field_vector_dict,
+                right_field_vector_dict,
+            ) = result
+        else:
+            left_vector_dict, right_vector_dict = result
+
         ann_index = ANNLinkageIndex(embedding_size=self.embedding_size)
         ann_index.insert_vector_dict(
             left_vector_dict=left_vector_dict, right_vector_dict=right_vector_dict
@@ -461,4 +573,8 @@ class LinkageEmbed(_BaseEmbed):
             left_source=self.left_source,
             index_search_kwargs=index_search_kwargs,
         )
-        return found_pair_set
+
+        if return_field_embeddings:
+            return found_pair_set, left_field_vector_dict, right_field_vector_dict
+        else:
+            return found_pair_set
